@@ -1,26 +1,59 @@
 # Delimiters used in PED files.
 delims <- "[ \t]"
 
-#' Creates a matrix wrapper around a binary PED file.
+#' Creates a matrix wrapper around binary PED files.
 #'
-#' The binary PED file will not be loaded into memory. Rather, subsets are
-#' extracted 'on the fly'.
+#' \code{BEDMatrix} is an S3 class that behaves similarly to a regular
+#' \code{matrix} by implementing key methods such as \code{[}, \code{dim}, and
+#' \code{dimnames}. Subsets are extracted directly and on-demand from the binary
+#' PED file without loading the entire file into memory through memory mapping.
 #'
-#' If a FAM or MAP file with the same name as the binary PED file (minus the
-#' extension) exists within the same folder, \code{n} and \code{p} as well as
-#' rownames and colnames will be automatically determined from those files.
-#' To skip this, provide \code{n} and \code{p} manually.
+#' A \code{BEDMatrix} instance can be created by providing the path to the BED
+#' file (with or without extension) as \code{path}, the number of individuals
+#' as \code{n}, and the number of markers as \code{p}. If a FAM file (which
+#' corresponds to the first six columns of a PED file) of the same name and in
+#' the same directory as the BED file exists, it is optional to provide
+#' \code{n} and the number of individuals as well as the rownames of the
+#' \code{BEDMatrix} will be detected automatically. If a BIM file (which
+#' corresponds to the MAP file that accompanies a PED file) of the same name
+#' and in the same directory as the BED file exists, it is optional to provide
+#' \code{p} and the number of markers as well as the colnames of the
+#' \code{BEDMatrix} will be detected automatically.  For very large BED file it
+#' is advised to provide \code{n} and \code{p} manually to speed up object
+#' creation. In that case \code{rownames} and \code{colnames} will be set to
+#' \code{NULL} and have to be specified manually.
 #'
-#' @param path Path to the binary PED file.
+#' A BED file can be created from a PED file with
+#' \href{http://pngu.mgh.harvard.edu/~purcell/plink/}{PLINK} using \code{plink
+#' --file myfile --make-bed}. BED files are storage and query efficient, and can
+#' be transformed back into the original PED file with PLINK using \code{plink
+#' --bfile myfile --recode}.
+#'
+#' Internally, \code{BEDMatrix} inherits from \code{list} and
+#' exposes a few attributes that should not be relied upon in actual code:
+#' \code{path}, \code{dims}, \code{dnames}, and \code{_instance}. \code{path}
+#' stores the path to the BED file. \code{dims} and \code{dnames} contain the
+#' dimensions and dimnames of the BEDMatrix object. \code{_instance} points to
+#' the underlying \code{Rcpp} module. The \code{Rcpp} module exposes an S4 class
+#' called \code{BEDMatrix_} that memory maps the BED file via
+#' \code{Boost.Interprocess} of the \code{BH} package.
+#'
+#' @param path Path to the binary PED file, with or without extension.
 #' @param n The number of individuals. Optional if FAM file of same name as BED
-#'   file exists.
+#'   file exists. If provided, \code{rownames} will be set to \code{NULL} and
+#'   have to be provided manually.
 #' @param p The number of markers. Optional if MAP file of same name as BED file
-#'   exists.
+#'   exists. If provided, \code{colnames} will be set to \code{NULL} and have
+#'   to be provided manually.
 #' @export
 BEDMatrix <- function(path, n = NULL, p = NULL) {
     path <- path.expand(path)
     if (!file.exists(path)) {
-        stop("File not found.")
+        # Try to add extension (common in PLINK).
+        path <- paste0(path, ".bed")
+        if (!file.exists(path)) {
+            stop("File not found.")
+        }
     }
     dir <- substr(path, 1, nchar(path) - 4)
     if (is.null(n)) {
@@ -68,6 +101,8 @@ BEDMatrix <- function(path, n = NULL, p = NULL) {
     class(s3_obj) <- "BEDMatrix"
     attr(s3_obj, "_instance") <- rcpp_obj
     attr(s3_obj, "dnames") <- list(rownames, colnames)
+    attr(s3_obj, "dims") <- c(rcpp_obj$n, rcpp_obj$p)
+    attr(s3_obj, "path") <- path
     return(s3_obj)
 }
 
@@ -138,16 +173,12 @@ print.BEDMatrix <- function(x, ...) {
 
 #' @export
 dim.BEDMatrix <- function(x) {
-    rcpp_obj <- attr(x, "_instance")
-    n <- rcpp_obj$n
-    p <- rcpp_obj$p
-    return(c(n, p))
+    attr(x, "dims")
 }
 
 #' @export
 dimnames.BEDMatrix <- function(x) {
-    dnames <- attr(x, "dnames")
-    return(dnames)
+    attr(x, "dnames")
 }
 
 #' @export
@@ -165,3 +196,18 @@ dimnames.BEDMatrix <- function(x) {
     })
     return(x)
 }
+
+#' @export
+length.BEDMatrix <- function(x) {
+    prod(dim(x))
+}
+
+#' @export
+is.matrix.BEDMatrix <- function(x) {
+    TRUE
+}
+
+#' @export
+as.matrix.BEDMatrix <- function(x, ...) {
+    x[, , drop = FALSE]
+}
